@@ -73,20 +73,19 @@ pbicop <- function(u, family, rotation, parameters) {
 
 #' @param n number of observations. If `length(n) > 1``, the length is taken to
 #'   be the number required.
-#' @param U optionally, an \eqn{n \times 2} matrix of values in \eqn{(0,1)}.
-#'    The result is then the inverse Rosenblatt transform of `U`; if `U` is a
-#'    matrix of independent \eqn{U(0, 1)} variables, this simulates data 
-#'    from `vinecop`.
+#' @param qrng if `TRUE`, generates quasi-random numbers using the bivariate 
+#' Generalized Halton sequence (default `qrng = FALSE`).
 #' @rdname bicop_methods
 #' @export
-rbicop <- function(n, family, rotation, parameters, U = NULL) {
+rbicop <- function(n, family, rotation, parameters, qrng = FALSE) {
     if (length(n) > 1)
         n <- length(n)
-    if (inherits(family, "bicop_dist") & is.null(U) & !missing(rotation))
-        U <- rotation
+    if (inherits(family, "bicop_dist") & !missing(rotation))
+        qrng <- rotation
+    assert_that(is.flag(qrng))
+
     bicop <- args2bicop(family, rotation, parameters)
-    U <- prep_uniform_data(n, 2, U)
-    U <- cbind(U[, 1], bicop_hinv1_cpp(U, bicop))
+    U <- bicop_sim_cpp(bicop, n, qrng, get_seeds())
     if (!is.null(bicop$names)) 
         colnames(U) <- bicop$names
     
@@ -166,10 +165,14 @@ ktau_to_par <- function(family, tau) {
 #' @return 
 #' `fitted()` and `logLik()` have return values similar to [dbicop()], 
 #' [pbicop()], and [hbicop()].
+#' 
+#' @details `fitted()` can only be called if the model was fit with the
+#'    `keep_data = TRUE` option.
+#' 
 #' @examples
 #' # Simulate and fit a bivariate copula model
 #' u <- rbicop(500, "gauss", 0, 0.5)
-#' fit <- bicop(u, "par")
+#' fit <- bicop(u, "par", keep_data = TRUE)
 #' 
 #' # Predictions
 #' all.equal(predict(fit, u, "hfunc1"), fitted(fit, "hfunc1"))
