@@ -62,7 +62,7 @@
 #' mat <- matrix(c(1, 2, 3, 1, 2, 0, 1, 0, 0), 3, 3) 
 #' 
 #' # set up vine copula model with Gaussian margins
-#' vc <- vine_dist(list(name = "norm"), pcs, mat)
+#' vc <- vine_dist(list(distr = "norm"), pcs, mat)
 #' 
 #' # show model
 #' summary(vc)
@@ -83,7 +83,7 @@ vine <- function(data,
                                       xmax = NaN, 
                                       bw = NA), 
                  copula_controls = list(family_set = "all", 
-                                     matrix = NA, 
+                                     structure = NA, 
                                      par_method = "mle", 
                                      nonpar_method = "constant",
                                      mult = 1, 
@@ -93,7 +93,7 @@ vine <- function(data,
                                      trunc_lvl = Inf, 
                                      tree_crit = "tau", 
                                      threshold = 0, 
-                                     keep_data = TRUE,
+                                     keep_data = FALSE,
                                      show_trace = FALSE, 
                                      cores = 1)) {
     
@@ -144,24 +144,27 @@ vine <- function(data,
 
 #' @param margins A list with with each element containing the specification of a 
 #' marginal [stats::Distributions]. Each marginal specification 
-#' should be a list with containing at least the name and optionally the 
-#' parameters, e.g. `list(list(name = "norm"), list(name = "norm", mu = 1), list(name = "beta", shape1 = 1, shape2 = 1))`.
+#' should be a list with containing at least the distribution family (`"distr"`) 
+#' and optionally the parameters, e.g. 
+#' `list(list(distr = "norm"), list(distr = "norm", mu = 1), list(distr = "beta", shape1 = 1, shape2 = 1))`.
 #' Note that parameters that have no default values have to be provided. 
 #' Furthermore, if `margins` has length one, it will be recycled for every component.
 #' @param pair_copulas A nested list of 'bicop_dist' objects, where 
 #'    \code{pair_copulas[[t]][[e]]} corresponds to the pair-copula at edge `e` in
 #'    tree `t`.
-#' @param matrix a quadratic matrix specifying the structure matrix (see 
-#'   [check_rvine_matrix()]); for [vinecop_dist()], the dimension must be 
-#'   `length(pair_copulas)-1`; for [vinecop()], `matrix = NA` performs
-#'   automatic structure selection.
+#' @param structure an `rvine_structure` object, namely a compressed 
+#' representation of the vine structure, or an object that can be coerced 
+#' into one (see [rvine_structure()] and [as_rvine_structure()]).
+#' The dimension must be `length(pair_copulas[[1]]) + 1`.
 #' @rdname vine
 #' @export
-vine_dist <- function(margins, pair_copulas, matrix) {
+vine_dist <- function(margins, pair_copulas, structure) {
+    
+    structure <- as_rvine_structure(structure)
     
     # sanity checks for the marg
-    if (!(length(margins) %in% c(1, ncol(matrix))))
-        stop("marg should have length 1 or ncol(matrix)")
+    if (!(length(margins) %in% c(1, dim(structure)[1])))
+        stop("marg should have length 1 or dim(structure)[1]")
     stopifnot(is.list(margins))
     if (depth(margins) == 1) {
         check_marg <- check_distr(margins)
@@ -179,7 +182,7 @@ vine_dist <- function(margins, pair_copulas, matrix) {
     }
 
     # create the vinecop object
-    copula <- vinecop_dist(pair_copulas, matrix)
+    copula <- vinecop_dist(pair_copulas, structure)
     
     # create object
     structure(list(margins = margins, 
@@ -233,7 +236,8 @@ finalize_vine <- function(vine, data, keep_data) {
     
     ## add number of observations
     vine$nobs <- nrow(data)
-    
+    vine$names <- vine$copula$names <- colnames(data)
+
     ## create and return object
     structure(vine, class = c("vine", "vine_dist"))
 }
\ No newline at end of file
