@@ -29,13 +29,15 @@ private:
 
     Eigen::Matrix2d select_bandwidth(
         const Eigen::Matrix<double, Eigen::Dynamic, 2> &x,
-        std::string method);
+        std::string method,
+        const Eigen::VectorXd& weights);
 
     Eigen::MatrixXd fit_local_likelihood(
         const Eigen::Matrix<double, Eigen::Dynamic, 2> &x,
         const Eigen::Matrix<double, Eigen::Dynamic, 2> &x_data,
         const Eigen::Matrix2d &B,
-        std::string method);
+        std::string method,
+        const Eigen::VectorXd& weights);
 
     double calculate_infl(const size_t &n,
                           const double &f0,
@@ -43,10 +45,13 @@ private:
                           const Eigen::Matrix2d &B,
                           const double &det_irB,
                           const Eigen::Matrix2d &S,
-                          const std::string &method);
+                          const std::string &method,
+                          const double& weight);
 
     void fit(const Eigen::Matrix<double, Eigen::Dynamic, 2> &data,
-             std::string method, double mult);
+             std::string method, 
+             double mult, 
+             const Eigen::VectorXd& weights);
 };
 }
 
